@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 import  html
@@ -85,7 +85,7 @@ class NoteImporter(Importer):
 
     def foreignNotes(self):
         "Return a list of foreign notes for importing."
-        assert 0
+        return []
 
     def open(self):
         "Open file and ensure it's in the right format."
