@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 import time
@@ -34,6 +34,7 @@ class Scheduler:
         self.reps = 0
         self.today = None
         self._haveQueues = False
+        self._lrnCutoff = 0
         self._updateCutoff()
 
     def getCard(self):
@@ -97,6 +98,11 @@ class Scheduler:
         else:
             assert 0
 
+        # once a card has been answered once, the original due date
+        # no longer applies
+        if card.odue:
+            card.odue = 0
+
     def _answerCardPreview(self, card, ease):
         assert 1 <= ease <= 2
 
@@ -205,7 +211,7 @@ order by due""" % self._deckLimit(),
     def deckDueList(self):
         "Returns [deckname, did, rev, lrn, new]"
         self._checkDay()
-        self.col.decks.recoverOrphans()
+        self.col.decks.checkIntegrity()
         decks = self.col.decks.all()
         decks.sort(key=itemgetter('name'))
         lims = {}
@@ -218,21 +224,10 @@ order by due""" % self._deckLimit(),
             return "::".join(parts)
         childMap = self.col.decks.childMap()
         for deck in decks:
-            # if we've already seen the exact same deck name, rename the
-            # invalid duplicate and reload
-            if deck['name'] in lims:
-                deck['name'] += "1"
-                self.col.decks.save(deck)
-                return self.deckDueList()
             p = parent(deck['name'])
             # new
             nlim = self._deckNewLimitSingle(deck)
             if p:
-                if p not in lims:
-                    # if parent was missing, this deck is invalid
-                    deck['name'] = "recovered"
-                    self.col.decks.save(deck)
-                    return self.deckDueList()
                 nlim = min(nlim, lims[p][0])
             new = self._newForDeck(deck['id'], nlim)
             # learning
@@ -364,7 +359,7 @@ did = ? and queue = 0 limit ?)""", did, lim)
             if lim:
                 # fill the queue with the current did
                 self._newQueue = self.col.db.list("""
-select id from cards where did = ? and queue = 0 order by due limit ?""", did, lim)
+                select id from cards where did = ? and queue = 0 order by due,ord limit ?""", did, lim)
                 if self._newQueue:
                     self._newQueue.reverse()
                     return True
@@ -444,15 +439,25 @@ select id from cards where did in %s and queue = 0 limit ?)"""
     # Learning queues
     ##########################################################################
 
-    def _resetLrnCount(self):
-        cutoff = intTime() + self.col.conf['collapseTime']
+    # scan for any newly due learning cards every minute
+    def _updateLrnCutoff(self, force):
+        nextCutoff = intTime() + self.col.conf['collapseTime']
+        if nextCutoff - self._lrnCutoff > 60 or force:
+            self._lrnCutoff = nextCutoff
+            return True
+        return False
 
+    def _maybeResetLrn(self, force):
+        if self._updateLrnCutoff(force):
+            self._resetLrn()
+
+    def _resetLrnCount(self):
         # sub-day
         self.lrnCount = self.col.db.scalar("""
 select count() from cards where did in %s and queue = 1
 and due < ?""" % (
             self._deckLimit()),
-            cutoff) or 0
+            self._lrnCutoff) or 0
         # day
         self.lrnCount += self.col.db.scalar("""
 select count() from cards where did in %s and queue = 3
@@ -464,6 +469,7 @@ select count() from cards where did in %s and queue = 4
 """ % (self._deckLimit()))
 
     def _resetLrn(self):
+        self._updateLrnCutoff(force=True)
         self._resetLrnCount()
         self._lrnQueue = []
         self._lrnDayQueue = []
@@ -485,6 +491,7 @@ limit %d""" % (self._deckLimit(), self.reportLimit), lim=cutoff)
         return self._lrnQueue
 
     def _getLrnCard(self, collapse=False):
+        self._maybeResetLrn(force=collapse and self.lrnCount == 0)
         if self._fillLrn():
             cutoff = time.time()
             if collapse:
@@ -656,7 +663,7 @@ did = ? and queue = 3 and due <= ? limit ?""",
         card.type = 2
 
     def _startingLeft(self, card):
-        if card.type == 2:
+        if card.type == 3:
             conf = self._lapseConf(card)
         else:
             conf = self._lrnConf(card)
@@ -785,19 +792,13 @@ did in %s and queue = 2 and due <= ? limit %d)""" % (
             self._revQueue = self.col.db.list("""
 select id from cards where
 did in %s and queue = 2 and due <= ?
-order by due
+order by due, random()
 limit ?""" % (ids2str(self.col.decks.active())),
                     self.today, lim)
 
             if self._revQueue:
-                if self.col.decks.get(self.col.decks.selected(), default=False)['dyn']:
-                    # dynamic decks need due order preserved
-                    self._revQueue.reverse()
-                else:
-                    # fixme: as soon as a card is answered, this is no longer consistent
-                    r = random.Random()
-                    r.seed(self.today)
-                    r.shuffle(self._revQueue)
+                # preserve order
+                self._revQueue.reverse()
                 return True
 
         if self.revCount:
@@ -857,7 +858,7 @@ select id from cards where did in %s and queue = 2 and due <= ? limit ?)"""
         return delay
 
     def _lapseIvl(self, card, conf):
-        ivl = max(1, conf['minInt'], card.ivl*conf['mult'])
+        ivl = max(1, conf['minInt'], int(card.ivl*conf['mult']))
         return ivl
 
     def _rescheduleRev(self, card, ease, early):
@@ -937,8 +938,8 @@ select id from cards where did in %s and queue = 2 and due <= ? limit ?)"""
         ivl = int(ivl * conf.get('ivlFct', 1))
         if fuzz:
             ivl = self._fuzzedIvl(ivl)
-        ivl = min(ivl, conf['maxIvl'])
         ivl = max(ivl, prev+1, 1)
+        ivl = min(ivl, conf['maxIvl'])
         return int(ivl)
 
     def _daysLate(self, card):
@@ -1028,10 +1029,9 @@ select id from cards where did in %s and queue = 2 and due <= ? limit ?)"""
             lim = "did = %s" % did
         self.col.log(self.col.db.list("select id from cards where %s" % lim))
 
-        # update queue in preview case
         self.col.db.execute("""
 update cards set did = odid, %s,
-due = odue, odue = 0, odid = 0, usn = ? where %s""" % (
+due = (case when odue>0 then odue else due end), odue = 0, odid = 0, usn = ? where %s""" % (
             self._restoreQueueSnippet, lim),
                             self.col.usn())
 
@@ -1196,7 +1196,7 @@ where id = ?
     def _updateCutoff(self):
         oldToday = self.today
         # days since col created
-        self.today = int((time.time() - self.col.crt) // 86400)
+        self.today = self._daysSinceCreation()
         # end of day cutoff
         self.dayCutoff = self._dayCutoff()
         if oldToday != self.today:
@@ -1230,9 +1230,15 @@ where id = ?
         if date < datetime.datetime.today():
             date = date + datetime.timedelta(days=1)
 
-        stamp = time.mktime(date.timetuple())
+        stamp = int(time.mktime(date.timetuple()))
         return stamp
 
+    def _daysSinceCreation(self):
+        startDate = datetime.datetime.fromtimestamp(self.col.crt)
+        startDate = startDate.replace(hour=self.col.conf.get("rollover", 4),
+                                      minute=0, second=0, microsecond=0)
+        return int((time.time() - time.mktime(startDate.timetuple())) // 86400)
+
     # Deck finished state
     ##########################################################################
 
@@ -1523,6 +1529,7 @@ usn=:usn,mod=:mod,factor=:fact where id=:id""",
             random.shuffle(nids)
         for c, nid in enumerate(nids):
             due[nid] = start+c*step
+        # pylint: disable=undefined-loop-variable
         high = start+c*step
         # shift?
         if shift:
@@ -1582,13 +1589,20 @@ else type end),
 due = odue, odue = 0, odid = 0, usn = ? where odid != 0""",
                             self.col.usn())
 
-    def _removeAllFromLearning(self):
+    def _removeAllFromLearning(self, schedVer=2):
         # remove review cards from relearning
-        self.col.db.execute("""
-update cards set
-due = odue, queue = 2, type = 2, mod = %d, usn = %d, odue = 0
-where queue in (1,3) and type in (2, 3)
-""" % (intTime(), self.col.usn()))
+        if schedVer == 1:
+            self.col.db.execute("""
+    update cards set
+    due = odue, queue = 2, type = 2, mod = %d, usn = %d, odue = 0
+    where queue in (1,3) and type in (2, 3)
+    """ % (intTime(), self.col.usn()))
+        else:
+            self.col.db.execute("""
+    update cards set
+    due = %d+ivl, queue = 2, type = 2, mod = %d, usn = %d, odue = 0
+    where queue in (1,3) and type in (2, 3)
+    """ % (self.today, intTime(), self.col.usn()))
         # remove new cards from learning
         self.forgetCards(self.col.db.list(
             "select id from cards where queue in (1,3)"))
@@ -1609,13 +1623,20 @@ where queue < 0""" % (intTime(), self.col.usn()))
     def _moveManuallyBuried(self):
         self.col.db.execute("update cards set queue=-2,mod=%d where queue=-3" % intTime())
 
+    # adding 'hard' in v2 scheduler means old ease entries need shifting
+    # up or down
+    def _remapLearningAnswers(self, sql):
+        self.col.db.execute("update revlog set %s and type in (0,2)" % sql)
+
     def moveToV1(self):
         self._emptyAllFiltered()
         self._removeAllFromLearning()
 
         self._moveManuallyBuried()
         self._resetSuspendedLearning()
+        self._remapLearningAnswers("ease=ease-1 where ease in (3,4)")
 
     def moveToV2(self):
         self._emptyAllFiltered()
-        self._removeAllFromLearning()
+        self._removeAllFromLearning(schedVer=1)
+        self._remapLearningAnswers("ease=ease+1 where ease in (2,3)")
