@@ -1,15 +1,18 @@
 # -*- coding: utf-8 -*-
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 import io
 import gzip
 import random
 import requests
+import json
+import os
 
 from anki.db import DB, DBError
-from anki.utils import ids2str, intTime, json, platDesc, checksum, devMode
+from anki.utils import ids2str, intTime, platDesc, checksum, devMode
 from anki.consts import *
+from anki.utils import versionWithBuild
 from .hooks import runHook
 import anki
 from .lang import ngettext
@@ -426,6 +429,7 @@ from notes where %s""" % d)
 class AnkiRequestsClient:
 
     verify = True
+    timeout = 60
 
     def __init__(self):
         self.session = requests.Session()
@@ -434,13 +438,13 @@ class AnkiRequestsClient:
         data = _MonitoringFile(data)
         headers['User-Agent'] = self._agentName()
         return self.session.post(
-            url, data=data, headers=headers, stream=True, timeout=60, verify=self.verify)
+            url, data=data, headers=headers, stream=True, timeout=self.timeout, verify=self.verify)
 
     def get(self, url, headers=None):
         if headers is None:
             headers = {}
         headers['User-Agent'] = self._agentName()
-        return self.session.get(url, stream=True, timeout=60, verify=self.verify)
+        return self.session.get(url, stream=True, headers=headers, timeout=self.timeout, verify=self.verify)
 
     def streamContent(self, resp):
         resp.raise_for_status()
@@ -584,7 +588,7 @@ class RemoteServer(HttpSyncer):
         )
         ret = self.req(
             "meta", io.BytesIO(json.dumps(dict(
-                v=SYNC_VER, cv="ankidesktop,%s,%s"%(anki.version, platDesc()))).encode("utf8")),
+                v=SYNC_VER, cv="ankidesktop,%s,%s"%(versionWithBuild(), platDesc()))).encode("utf8")),
             badAuthRaises=False)
         if not ret:
             # invalid auth
@@ -634,6 +638,7 @@ class FullSyncer(HttpSyncer):
 
     def download(self):
         runHook("sync", "download")
+        localNotEmpty = self.col.db.scalar("select 1 from cards")
         self.col.close()
         cont = self.req("download")
         tpath = self.col.path + ".tmp"
@@ -644,7 +649,12 @@ class FullSyncer(HttpSyncer):
         # check the received file is ok
         d = DB(tpath)
         assert d.scalar("pragma integrity_check") == "ok"
+        remoteEmpty = not d.scalar("select 1 from cards")
         d.close()
+        # accidental clobber?
+        if localNotEmpty and remoteEmpty:
+            os.unlink(tpath)
+            return "downloadClobber"
         # overwrite existing collection
         os.unlink(self.col.path)
         os.rename(tpath, self.col.path)
@@ -725,7 +735,8 @@ class MediaSyncer:
                         need.append(fname)
                     else:
                         self.col.log("have same already")
-                    ldirty and self.col.media.markClean([fname])
+                    if ldirty:
+                        self.col.media.markClean([fname])
                 elif lsum:
                     # deleted remotely
                     if not ldirty:
@@ -737,7 +748,8 @@ class MediaSyncer:
                 else:
                     # deleted both sides
                     self.col.log("both sides deleted")
-                    ldirty and self.col.media.markClean([fname])
+                    if ldirty:
+                        self.col.media.markClean([fname])
 
             self._downloadFiles(need)
 
