@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 import re
@@ -17,10 +17,12 @@ import locale
 from hashlib import sha1
 import platform
 import traceback
-import json
-
+from contextlib import contextmanager
 from anki.lang import _, ngettext
 
+# some add-ons expect json to be in the utils module
+import json # pylint: disable=unused-import
+
 # Time handling
 ##############################################################################
 
@@ -37,13 +39,13 @@ timeTable = {
     "seconds": lambda n: ngettext("%s second", "%s seconds", n),
     }
 
-afterTimeTable = {
-    "years": lambda n: ngettext("%s year<!--after-->", "%s years<!--after-->", n),
-    "months": lambda n: ngettext("%s month<!--after-->", "%s months<!--after-->", n),
-    "days": lambda n: ngettext("%s day<!--after-->", "%s days<!--after-->", n),
-    "hours": lambda n: ngettext("%s hour<!--after-->", "%s hours<!--after-->", n),
-    "minutes": lambda n: ngettext("%s minute<!--after-->", "%s minutes<!--after-->", n),
-    "seconds": lambda n: ngettext("%s second<!--after-->", "%s seconds<!--after-->", n),
+inTimeTable = {
+    "years": lambda n: ngettext("in %s year", "in %s years", n),
+    "months": lambda n: ngettext("in %s month", "in %s months", n),
+    "days": lambda n: ngettext("in %s day", "in %s days", n),
+    "hours": lambda n: ngettext("in %s hour", "in %s hours", n),
+    "minutes": lambda n: ngettext("in %s minute", "in %s minutes", n),
+    "seconds": lambda n: ngettext("in %s second", "in %s seconds", n),
     }
 
 def shortTimeFmt(type):
@@ -56,7 +58,7 @@ def shortTimeFmt(type):
     "seconds": _("%ss"),
     }[type]
 
-def fmtTimeSpan(time, pad=0, point=0, short=False, after=False, unit=99):
+def fmtTimeSpan(time, pad=0, point=0, short=False, inTime=False, unit=99):
     "Return a string representing a time span (eg '2 days')."
     (type, point) = optimalPeriod(time, point, unit)
     time = convertSecondsTo(time, type)
@@ -65,12 +67,12 @@ def fmtTimeSpan(time, pad=0, point=0, short=False, after=False, unit=99):
     if short:
         fmt = shortTimeFmt(type)
     else:
-        if after:
-            fmt = afterTimeTable[type](_pluralCount(time, point))
+        if inTime:
+            fmt = inTimeTable[type](_pluralCount(time, point))
         else:
             fmt = timeTable[type](_pluralCount(time, point))
-    timestr = "%(a)d.%(b)df" % {'a': pad, 'b': point}
-    return locale.format_string("%" + (fmt % timestr), time)
+    timestr = "%%%(a)d.%(b)df" % {'a': pad, 'b': point}
+    return locale.format_string(fmt % timestr, time)
 
 def optimalPeriod(time, point, unit):
     if abs(time) < 60 or unit < 1:
@@ -125,13 +127,15 @@ def fmtFloat(float_value, point=1):
 
 # HTML
 ##############################################################################
+reComment = re.compile("(?s)<!--.*?-->")
 reStyle = re.compile("(?si)<style.*?>.*?</style>")
 reScript = re.compile("(?si)<script.*?>.*?</script>")
-reTag = re.compile("<.*?>")
-reEnts = re.compile("&#?\w+;")
+reTag = re.compile("(?s)<.*?>")
+reEnts = re.compile(r"&#?\w+;")
 reMedia = re.compile("(?i)<img[^>]+src=[\"']?([^\"'>]+)[\"']?[^>]*>")
 
 def stripHTML(s):
+    s = reComment.sub("", s)
     s = reStyle.sub("", s)
     s = reScript.sub("", s)
     s = reTag.sub("", s)
@@ -153,6 +157,17 @@ def minimizeHTML(s):
                '<u>\\1</u>', s)
     return s
 
+def htmlToTextLine(s):
+    s = s.replace("<br>", " ")
+    s = s.replace("<br />", " ")
+    s = s.replace("<div>", " ")
+    s = s.replace("\n", " ")
+    s = re.sub(r"\[sound:[^]]+\]", "", s)
+    s = re.sub(r"\[\[type:[^]]+\]\]", "", s)
+    s = stripHTMLMedia(s)
+    s = s.strip()
+    return s
+
 def entsToTxt(html):
     # entitydefs defines nbsp as \xa0 instead of a standard space, so we
     # replace it first
@@ -177,6 +192,12 @@ def entsToTxt(html):
         return text # leave as is
     return reEnts.sub(fixup, html)
 
+def bodyClass(col, card):
+    bodyclass = "card card%d" % (card.ord+1)
+    if col.conf.get("nightMode"):
+        bodyclass += " nightMode"
+    return bodyclass
+
 # IDs
 ##############################################################################
 
@@ -295,6 +316,13 @@ def namedtmp(name, rm=True):
 # Cmd invocation
 ##############################################################################
 
+@contextmanager
+def noBundledLibs():
+    oldlpath = os.environ.pop("LD_LIBRARY_PATH", None)
+    yield
+    if oldlpath is not None:
+        os.environ["LD_LIBRARY_PATH"] = oldlpath
+
 def call(argv, wait=True, **kwargs):
     "Execute a command. If WAIT, return exit code."
     # ensure we don't open a separate window for forking process on windows
@@ -303,12 +331,14 @@ def call(argv, wait=True, **kwargs):
         try:
             si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
         except:
+            # pylint: disable=no-member
             si.dwFlags |= subprocess._subprocess.STARTF_USESHOWWINDOW
     else:
         si = None
     # run
     try:
-        o = subprocess.Popen(argv, startupinfo=si, **kwargs)
+        with noBundledLibs():
+            o = subprocess.Popen(argv, startupinfo=si, **kwargs)
     except OSError:
         # command not found
         return -1
@@ -331,6 +361,7 @@ def call(argv, wait=True, **kwargs):
 isMac = sys.platform.startswith("darwin")
 isWin = sys.platform.startswith("win32")
 isLin = not isMac and not isWin
+devMode = os.getenv("ANKIDEV", "")
 
 invalidFilenameChars = ":*?\"<>|"
 
@@ -358,8 +389,9 @@ def platDesc():
             elif isWin:
                 theos = "win:%s" % (platform.win32_ver()[0])
             elif system == "Linux":
-                dist = platform.dist()
-                theos = "lin:%s:%s" % (dist[0], dist[1])
+                import distro
+                r = distro.linux_distribution(full_distribution_name=False)
+                theos = "lin:%s:%s" % (r[0], r[1])
             else:
                 theos = system
             break
@@ -378,3 +410,13 @@ class TimedLog:
         sys.stderr.write("%5dms: %s(): %s\n" % ((time.time() - self._last)*1000, fn, s))
         self._last = time.time()
 
+# Version
+##############################################################################
+
+def versionWithBuild():
+    from anki import version
+    try:
+        from anki.buildhash import build
+    except:
+        build = "dev"
+    return "%s (%s)" % (version, build)
