@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 import sre_constants
 import html
 import time
 import re
+import unicodedata
 from operator import  itemgetter
 from anki.lang import ngettext
 import json
@@ -13,17 +14,18 @@ import json
 from aqt.qt import *
 import anki
 import aqt.forms
-from anki.utils import fmtTimeSpan, ids2str, stripHTMLMedia, htmlToTextLine, \
+from anki.utils import fmtTimeSpan, ids2str, htmlToTextLine, \
     isWin, intTime, \
-    isMac, isLin, bodyClass
+    isMac, bodyClass
 from aqt.utils import saveGeom, restoreGeom, saveSplitter, restoreSplitter, \
-    saveHeader, restoreHeader, saveState, restoreState, applyStyles, getTag, \
+    saveHeader, restoreHeader, saveState, restoreState, getTag, \
     showInfo, askUser, tooltip, openHelp, showWarning, shortcut, mungeQA, \
-    getOnlyText, MenuList, SubMenu
+    getOnlyText, MenuList, SubMenu, qtMenuShortcutWorkaround
+from anki.lang import _
 from anki.hooks import runHook, addHook, remHook, runFilter
 from aqt.webview import AnkiWebView
 from anki.consts import *
-from anki.sound import playFromText, clearAudioQueue, allSounds, play
+from anki.sound import clearAudioQueue, allSounds, play
 
 
 # Data model
@@ -207,11 +209,13 @@ class DataModel(QAbstractTableModel):
         tv = self.browser.form.tableView
         if idx:
             tv.selectRow(idx.row())
-            # we save and then restore the horizontal scroll position because
-            # scrollTo() also scrolls horizontally which is confusing
-            h = tv.horizontalScrollBar().value()
-            tv.scrollTo(idx, tv.PositionAtCenter)
-            tv.horizontalScrollBar().setValue(h)
+            # scroll if the selection count has changed
+            if count != len(self.selectedCards):
+                # we save and then restore the horizontal scroll position because
+                # scrollTo() also scrolls horizontally which is confusing
+                h = tv.horizontalScrollBar().value()
+                tv.scrollTo(idx, tv.PositionAtCenter)
+                tv.horizontalScrollBar().setValue(h)
             if count < 500:
                 # discard large selections; they're too slow
                 sm.select(items, QItemSelectionModel.SelectCurrent |
@@ -312,6 +316,17 @@ class DataModel(QAbstractTableModel):
             return ""
         return time.strftime("%Y-%m-%d", time.localtime(date))
 
+    def isRTL(self, index):
+        col = index.column()
+        type = self.columnType(col)
+        if type != "noteFld":
+            return False
+
+        row = index.row()
+        c = self.getCard(index)
+        nt = c.note().model()
+        return nt['flds'][self.col.models.sortIdx(nt)]['rtl']
+
 # Line painter
 ######################################################################
 
@@ -319,8 +334,8 @@ COLOUR_SUSPENDED = "#FFFFB2"
 COLOUR_MARKED = "#ccc"
 
 flagColours = {
-    1: "#F5B7B1",
-    2: "#BB8FCE",
+    1: "#ffaaaa",
+    2: "#ffb347",
     3: "#82E0AA",
     4: "#85C1E9",
 }
@@ -343,13 +358,16 @@ class StatusDelegate(QItemDelegate):
         finally:
             self.browser.mw.progress.blockUpdates = True
 
+        if self.model.isRTL(index):
+            option.direction = Qt.RightToLeft
+
         col = None
-        if c.queue == -1:
-            col = COLOUR_SUSPENDED
-        elif c.userFlag() > 0:
+        if c.userFlag() > 0:
             col = flagColours[c.userFlag()]
         elif c.note().hasTag("Marked"):
             col = COLOUR_MARKED
+        elif c.queue == -1:
+            col = COLOUR_SUSPENDED
         if col:
             brush = QBrush(QColor(col))
             painter.save()
@@ -367,7 +385,6 @@ class Browser(QMainWindow):
 
     def __init__(self, mw):
         QMainWindow.__init__(self, None, Qt.Window)
-        applyStyles(self)
         self.mw = mw
         self.col = self.mw.col
         self.lastFilter = ""
@@ -394,6 +411,7 @@ class Browser(QMainWindow):
         self.show()
 
     def setupMenus(self):
+        # pylint: disable=unnecessary-lambda
         # actions
         f = self.form
         f.previewButton.clicked.connect(self.onTogglePreview)
@@ -425,10 +443,9 @@ class Browser(QMainWindow):
         f.actionReschedule.triggered.connect(self.reschedule)
         f.actionToggle_Suspend.triggered.connect(self.onSuspend)
         f.actionRed_Flag.triggered.connect(lambda: self.onSetFlag(1))
-        f.actionPurple_Flag.triggered.connect(lambda: self.onSetFlag(2))
+        f.actionOrange_Flag.triggered.connect(lambda: self.onSetFlag(2))
         f.actionGreen_Flag.triggered.connect(lambda: self.onSetFlag(3))
         f.actionBlue_Flag.triggered.connect(lambda: self.onSetFlag(4))
-        f.actionClear_Flag.triggered.connect(lambda: self.onSetFlag(0))
         # jumps
         f.actionPreviousCard.triggered.connect(self.onPreviousCard)
         f.actionNextCard.triggered.connect(self.onNextCard)
@@ -462,6 +479,8 @@ class Browser(QMainWindow):
         for act in self.form.menu_Notes.actions():
             m.addAction(act)
         runHook("browser.onContextMenu", self, m)
+
+        qtMenuShortcutWorkaround(m)
         m.exec_(QCursor.pos())
 
     def updateFont(self):
@@ -556,8 +575,11 @@ class Browser(QMainWindow):
         if self.form.searchEdit.lineEdit().text() == self._searchPrompt:
             self.form.searchEdit.lineEdit().setText("deck:current ")
 
+        # grab search text and normalize
+        txt = self.form.searchEdit.lineEdit().text()
+        txt = unicodedata.normalize("NFC", txt)
+
         # update history
-        txt = str(self.form.searchEdit.lineEdit().text())
         sh = self.mw.pm.profile['searchHistory']
         if txt in sh:
             sh.remove(txt)
@@ -612,6 +634,8 @@ class Browser(QMainWindow):
         self.form.tableView.selectionModel()
         self.form.tableView.setItemDelegate(StatusDelegate(self, self.model))
         self.form.tableView.selectionModel().selectionChanged.connect(self.onRowChanged)
+        self.form.tableView.setStyleSheet("QTableView{ selection-background-color: rgba(127, 127, 127, 50);  }")
+        self.singleCard = False
 
     def setupEditor(self):
         self.editor = aqt.editor.Editor(
@@ -637,12 +661,17 @@ class Browser(QMainWindow):
             self.focusTo = None
             self.editor.card = self.card
             self.singleCard = True
+        self._updateFlagsMenu()
+        runHook("browser.rowChanged", self)
         self._renderPreview(True)
 
     def refreshCurrentCard(self, note):
         self.model.refreshNote(note)
         self._renderPreview(False)
 
+    def onLoadNote(self, editor):
+        self.refreshCurrentCard(editor.note)
+
     def refreshCurrentCardFilter(self, flag, note, fidx):
         self.refreshCurrentCard(note)
         return flag
@@ -778,8 +807,8 @@ by clicking on one on the left."""))
         def __init__(self):
             QTreeWidget.__init__(self)
             self.itemClicked.connect(self.onTreeClick)
-            self.itemExpanded.connect(lambda item: self.onTreeCollapse(item))
-            self.itemCollapsed.connect(lambda item: self.onTreeCollapse(item))
+            self.itemExpanded.connect(self.onTreeCollapse)
+            self.itemCollapsed.connect(self.onTreeCollapse)
 
         def keyPressEvent(self, evt):
             if evt.key() in (Qt.Key_Return, Qt.Key_Enter):
@@ -808,7 +837,6 @@ by clicking on one on the left."""))
         p = QPalette()
         p.setColor(QPalette.Base, p.window().color())
         self.sidebarTree.setPalette(p)
-        self.sidebarDockWidget.setVisible(False)
         self.sidebarDockWidget.setFloating(False)
         self.sidebarDockWidget.visibilityChanged.connect(self.onSidebarVisChanged)
         self.sidebarDockWidget.setTitleBarWidget(QWidget())
@@ -923,7 +951,7 @@ by clicking on one on the left."""))
         if self.mw.app.keyboardModifiers() & Qt.ControlModifier:
             cur = str(self.form.searchEdit.lineEdit().text())
             if cur and cur != self._searchPrompt:
-                        txt = cur + " " + txt
+                txt = cur + " " + txt
         elif self.mw.app.keyboardModifiers() & Qt.ShiftModifier:
             cur = str(self.form.searchEdit.lineEdit().text())
             if cur:
@@ -969,7 +997,7 @@ by clicking on one on the left."""))
             (_("Buried"), "is:buried"),
             None,
             (_("Red Flag"), "flag:1"),
-            (_("Purple Flag"), "flag:2"),
+            (_("Orange Flag"), "flag:2"),
             (_("Green Flag"), "flag:3"),
             (_("Blue Flag"), "flag:4"),
             (_("No Flag"), "flag:0"),
@@ -1099,6 +1127,8 @@ by clicking on one on the left."""))
         info, cs = self._cardInfoData()
         reps = self._revlogData(cs)
         class CardInfoDialog(QDialog):
+            silentlyClose = True
+
             def reject(self):
                 saveGeom(self, "revlog")
                 return QDialog.reject(self)
@@ -1225,6 +1255,7 @@ where id in %s""" % ids2str(sf))
 
     _previewTimer = None
     _lastPreviewRender = 0
+    _lastPreviewState = None
 
     def onTogglePreview(self):
         if self._previewWindow:
@@ -1234,6 +1265,7 @@ where id in %s""" % ids2str(sf))
 
     def _openPreview(self):
         self._previewState = "question"
+        self._lastPreviewState = None
         self._previewWindow = QDialog(None, Qt.Window)
         self._previewWindow.setWindowTitle(_("Preview"))
 
@@ -1268,9 +1300,9 @@ where id in %s""" % ids2str(sf))
         self.previewShowBothSides.setShortcut(QKeySequence("B"))
         self.previewShowBothSides.setToolTip(_("Shortcut key: %s" % "B"))
         bbox.addButton(self.previewShowBothSides, QDialogButtonBox.ActionRole)
-        self.previewShowBothSides.toggled.connect(self._onPreviewShowBothSides)
         self._previewBothSides = self.col.conf.get("previewBothSides", False)
         self.previewShowBothSides.setChecked(self._previewBothSides)
+        self.previewShowBothSides.toggled.connect(self._onPreviewShowBothSides)
 
         self._setupPreviewWebview()
 
@@ -1356,16 +1388,24 @@ where id in %s""" % ids2str(sf))
         if not c or not self.singleCard:
             txt = _("(please select 1 card)")
             bodyclass = ""
+            self._lastPreviewState = None
         else:
+            if self._previewBothSides:
+                self._previewState = "answer"
+            elif cardChanged:
+                self._previewState = "question"
+
+            currentState = self._previewStateAndMod()
+            if currentState == self._lastPreviewState:
+                # nothing has changed, avoid refreshing
+                return
+
             # need to force reload even if answer
             txt = c.q(reload=True)
 
             questionAudio = []
             if self._previewBothSides:
-                self._previewState = "answer"
                 questionAudio = allSounds(txt)
-            elif cardChanged:
-                self._previewState = "question"
             if self._previewState == "answer":
                 func = "_showAnswer"
                 txt = c.a()
@@ -1386,7 +1426,7 @@ where id in %s""" % ids2str(sf))
             txt = mungeQA(self.col, txt)
             txt = runFilter("prepareQA", txt, c,
                             "preview"+self._previewState.capitalize())
-
+            self._lastPreviewState = self._previewStateAndMod()
         self._updatePreviewButtons()
         self._previewWeb.eval(
             "{}({},'{}');".format(func, json.dumps(txt), bodyclass))
@@ -1399,6 +1439,12 @@ where id in %s""" % ids2str(sf))
             self._previewState = "question"
         self._renderPreview()
 
+    def _previewStateAndMod(self):
+        c = self.card
+        n = c.note()
+        n.load()
+        return (self._previewState, c.id, n.mod)
+
     # Card deletion
     ######################################################################
 
@@ -1538,9 +1584,27 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
     ######################################################################
 
     def onSetFlag(self, n):
+        # flag needs toggling off?
+        if n == self.card.userFlag():
+            n = 0
         self.col.setUserFlag(n, self.selectedCards())
         self.model.reset()
 
+    def _updateFlagsMenu(self):
+        flag = self.card and self.card.userFlag()
+        flag = flag or 0
+
+        f = self.form
+        flagActions = [f.actionRed_Flag,
+                       f.actionOrange_Flag,
+                       f.actionGreen_Flag,
+                       f.actionBlue_Flag]
+
+        for c, act in enumerate(flagActions):
+            act.setChecked(flag == c+1)
+
+        qtMenuShortcutWorkaround(self.form.menuFlag)
+
     def onMark(self, mark=None):
         if mark is None:
             mark = not self.isMarked()
@@ -1643,6 +1707,7 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
         addHook("undoState", self.onUndoState)
         addHook("reset", self.onReset)
         addHook("editTimer", self.refreshCurrentCard)
+        addHook("loadNote", self.onLoadNote)
         addHook("editFocusLost", self.refreshCurrentCardFilter)
         for t in "newTag", "newModel", "newDeck":
             addHook(t, self.maybeRefreshSidebar)
@@ -1650,6 +1715,7 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
     def teardownHooks(self):
         remHook("reset", self.onReset)
         remHook("editTimer", self.refreshCurrentCard)
+        remHook("loadNote", self.onLoadNote)
         remHook("editFocusLost", self.refreshCurrentCardFilter)
         remHook("undoState", self.onUndoState)
         for t in "newTag", "newModel", "newDeck":
@@ -1671,7 +1737,7 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
         if not sf:
             return
         import anki.find
-        fields = sorted(anki.find.fieldNames(self.col, downcase=False))
+        fields = anki.find.fieldNamesForNotes(self.mw.col, sf)
         d = QDialog(self)
         frm = aqt.forms.findreplace.Ui_Dialog()
         frm.setupUi(d)
@@ -1728,7 +1794,8 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
         frm = aqt.forms.finddupes.Ui_Dialog()
         frm.setupUi(d)
         restoreGeom(d, "findDupes")
-        fields = sorted(anki.find.fieldNames(self.col, downcase=False))
+        fields = sorted(anki.find.fieldNames(self.col, downcase=False),
+                        key=lambda x: x.lower())
         frm.fields.addItems(fields)
         self._dupesButton = None
         # links
@@ -2012,7 +2079,6 @@ Any cards mapped to nothing will be deleted. \
 If a note has no remaining cards, it will be lost. \
 Are you sure you want to continue?""")):
                 return
-        QDialog.accept(self)
         self.browser.mw.checkpoint(_("Change Note Type"))
         b = self.browser
         b.mw.col.modSchema(check=True)
@@ -2025,6 +2091,7 @@ Are you sure you want to continue?""")):
         b.mw.progress.finish()
         b.mw.reset()
         self.cleanup()
+        QDialog.accept(self)
 
     def onHelp(self):
         openHelp("browsermisc")
