@@ -1,4 +1,4 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # -*- coding: utf-8 -*-
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
@@ -23,13 +23,13 @@ import aqt.webview
 import aqt.toolbar
 import aqt.stats
 import aqt.mediasrv
-from aqt.utils import showWarning
 import anki.sound
 import anki.mpv
 from aqt.utils import saveGeom, restoreGeom, showInfo, showWarning, \
     restoreState, getOnlyText, askUser, showText, tooltip, \
     openHelp, openLink, checkInvalidFilename, getFile
-import sip
+from aqt.qt import sip
+from anki.lang import _, ngettext
 
 class AnkiQt(QMainWindow):
     def __init__(self, app, profileManager, opts, args):
@@ -39,12 +39,6 @@ class AnkiQt(QMainWindow):
         aqt.mw = self
         self.app = app
         self.pm = profileManager
-        # running 2.0 for the first time?
-        if self.pm.meta['firstRun']:
-            # load the new deck user profile
-            self.pm.load(self.pm.profiles()[0])
-            self.pm.meta['firstRun'] = False
-            self.pm.save()
         # init rest of app
         self.safeMode = self.app.queryKeyboardModifiers() & Qt.ShiftModifier
         try:
@@ -62,7 +56,11 @@ class AnkiQt(QMainWindow):
             self.onAppMsg(args[0])
         # Load profile in a timer so we can let the window finish init and not
         # close on profile load error.
-        self.progress.timer(10, self.setupProfile, False)
+        if isWin:
+            fn = self.setupProfileAfterWebviewsLoaded
+        else:
+            fn = self.setupProfile
+        self.progress.timer(10, fn, False, requiresCollection=False)
 
     def setupUI(self):
         self.col = None
@@ -73,6 +71,7 @@ class AnkiQt(QMainWindow):
         self.setupThreads()
         self.setupMediaServer()
         self.setupSound()
+        self.setupSpellCheck()
         self.setupMainWindow()
         self.setupSystemSpecific()
         self.setupStyle()
@@ -89,6 +88,16 @@ class AnkiQt(QMainWindow):
         self.setupOverview()
         self.setupReviewer()
 
+    def setupProfileAfterWebviewsLoaded(self):
+        for w in (self.web, self.bottomWeb):
+            if not w._domDone:
+                self.progress.timer(10, self.setupProfileAfterWebviewsLoaded, False, requiresCollection=False)
+                return
+            else:
+                w.requiresCol = True
+
+        self.setupProfile()
+
     # Profiles
     ##########################################################################
 
@@ -107,6 +116,12 @@ class AnkiQt(QMainWindow):
             self.closeFires = True
 
     def setupProfile(self):
+        if self.pm.meta['firstRun']:
+            # load the new deck user profile
+            self.pm.load(self.pm.profiles()[0])
+            self.pm.meta['firstRun'] = False
+            self.pm.save()
+
         self.pendingImport = None
         self.restoringBackup = False
         # profile not provided on command line?
@@ -174,9 +189,8 @@ class AnkiQt(QMainWindow):
         return not checkInvalidFilename(str)
 
     def onAddProfile(self):
-        name = getOnlyText(_("Name:"))
+        name = getOnlyText(_("Name:")).strip()
         if name:
-            name = name.strip()
             if name in self.pm.profiles():
                 return showWarning(_("Name exists."))
             if not self.profileNameOk(name):
@@ -186,7 +200,7 @@ class AnkiQt(QMainWindow):
             self.refreshProfilesList()
 
     def onRenameProfile(self):
-        name = getOnlyText(_("New name:"), default=self.pm.name)
+        name = getOnlyText(_("New name:"), default=self.pm.name).strip()
         if not name:
             return
         if name == self.pm.name:
@@ -291,7 +305,7 @@ close the profile or restart Anki."""))
                 if getattr(w, "silentlyClose", None):
                     w.close()
                 else:
-                    showWarning("Window should have been closed: {}".format(w))
+                    print("Window should have been closed: {}".format(w))
 
     def unloadProfileAndExit(self):
         self.unloadProfile(self.cleanupAndExit)
@@ -341,6 +355,7 @@ Debug info:
                 self.col = None
 
             # return to profile manager
+            self.hide()
             self.showProfileManager()
             return False
 
@@ -462,6 +477,7 @@ from the profile screen."))
         oldState = self.state or "dummy"
         cleanup = getattr(self, "_"+oldState+"Cleanup", None)
         if cleanup:
+            # pylint: disable=not-callable
             cleanup(state)
         self.clearStateShortcuts()
         self.state = state
@@ -605,6 +621,13 @@ title="%s" %s>%s</button>''' % (
         self.mainLayout.addWidget(sweb)
         self.form.centralwidget.setLayout(self.mainLayout)
 
+        # force webengine processes to load before cwd is changed
+        if isWin:
+            for o in self.web, self.bottomWeb:
+                o.requiresCol = False
+                o._domReady = False
+                o._page.setContent(bytes("", "ascii"))
+
     def closeAllWindows(self, onsuccess):
         aqt.dialogs.closeAll(onsuccess)
 
@@ -636,6 +659,10 @@ title="%s" %s>%s</button>''' % (
         if not self.safeMode:
             self.addonManager.loadAddons()
 
+    def setupSpellCheck(self):
+        os.environ["QTWEBENGINE_DICTIONARIES_PATH"] = (
+            os.path.join(self.pm.base, "dictionaries"))
+
     def setupThreads(self):
         self._mainThread = QThread.currentThread()
 
@@ -693,9 +720,6 @@ title="%s" %s>%s</button>''' % (
             self.setWindowState(self.windowState() & ~Qt.WindowMinimized)
         return True
 
-    def setStatus(self, text, timeout=3000):
-        self.form.statusbar.showMessage(text, timeout)
-
     def setupStyle(self):
         buf = ""
 
@@ -790,10 +814,14 @@ QTreeWidget {
         cid = self.col.undo()
         if cid and self.state == "review":
             card = self.col.getCard(cid)
+            self.col.sched.reset()
             self.reviewer.cardQueue.append(card)
+            self.reviewer.nextCard()
+            runHook("revertedCard", cid)
         else:
+            self.reset()
             tooltip(_("Reverted to state prior to '%s'.") % n.lower())
-        self.reset()
+            runHook("revertedState", n)
         self.maybeEnableUndo()
 
     def maybeEnableUndo(self):
@@ -1120,10 +1148,21 @@ will be lost. Continue?"""))
             _("Delete unused media?")):
             return
         mdir = self.col.media.dir()
-        for f in unused:
-            path = os.path.join(mdir, f)
-            if os.path.exists(path):
-                send2trash(path)
+        self.progress.start(immediate=True)
+        try:
+            lastProgress = 0
+            for c, f in enumerate(unused):
+                path = os.path.join(mdir, f)
+                if os.path.exists(path):
+                    send2trash(path)
+
+                now = time.time()
+                if now - lastProgress >= 0.3:
+                    lastProgress = now
+                    label = _("Deleted %s files...") % (c+1)
+                    self.progress.update(label)
+        finally:
+            self.progress.finish()
         tooltip(_("Deleted."))
         diag.close()
 
@@ -1168,11 +1207,19 @@ will be lost. Continue?"""))
         d.silentlyClose = True
         frm = aqt.forms.debug.Ui_Dialog()
         frm.setupUi(d)
+        font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
+        font.setPointSize(frm.text.font().pointSize() + 1)
+        frm.text.setFont(font)
+        frm.log.setFont(font)
         s = self.debugDiagShort = QShortcut(QKeySequence("ctrl+return"), d)
         s.activated.connect(lambda: self.onDebugRet(frm))
         s = self.debugDiagShort = QShortcut(
             QKeySequence("ctrl+shift+return"), d)
         s.activated.connect(lambda: self.onDebugPrint(frm))
+        s = self.debugDiagShort = QShortcut(QKeySequence("ctrl+l"), d)
+        s.activated.connect(frm.log.clear)
+        s = self.debugDiagShort = QShortcut(QKeySequence("ctrl+shift+l"), d)
+        s.activated.connect(frm.text.clear)
         d.show()
 
     def _captureOutput(self, on):
@@ -1198,7 +1245,16 @@ will be lost. Continue?"""))
         return aqt.dialogs._dialogs['Browser'][1].card.__dict__
 
     def onDebugPrint(self, frm):
-        frm.text.setPlainText("pp(%s)" % frm.text.toPlainText())
+        cursor = frm.text.textCursor()
+        position = cursor.position()
+        cursor.select(QTextCursor.LineUnderCursor)
+        line = cursor.selectedText()
+        pfx, sfx = "pp(", ")"
+        if not line.startswith(pfx):
+            line = "{}{}{}".format(pfx, line, sfx)
+            cursor.insertText(line)
+            cursor.setPosition(position + len(pfx))
+            frm.text.setTextCursor(cursor)
         self.onDebugRet(frm)
 
     def onDebugRet(self, frm):
@@ -1210,6 +1266,7 @@ will be lost. Continue?"""))
         pp = pprint.pprint
         self._captureOutput(True)
         try:
+            # pylint: disable=exec-used
             exec(text)
         except:
             self._output += traceback.format_exc()
@@ -1270,7 +1327,8 @@ will be lost. Continue?"""))
     def onAppMsg(self, buf):
         if self.state == "startup":
             # try again in a second
-            return self.progress.timer(1000, lambda: self.onAppMsg(buf), False)
+            return self.progress.timer(1000, lambda: self.onAppMsg(buf), False,
+                                       requiresCollection=False)
         elif self.state == "profileManager":
             # can't raise window while in profile manager
             if buf == "raise":
@@ -1308,7 +1366,7 @@ Please ensure a profile is open and Anki is not busy, then try again."""),
 
     def gcWindow(self, obj):
         obj.deleteLater()
-        self.progress.timer(1000, self.doGC, False)
+        self.progress.timer(1000, self.doGC, False, requiresCollection=False)
 
     def disableGC(self):
         gc.collect()
@@ -1330,7 +1388,7 @@ Please ensure a profile is open and Anki is not busy, then try again."""),
     ##########################################################################
 
     def setupMediaServer(self):
-        self.mediaServer = aqt.mediasrv.MediaServer()
+        self.mediaServer = aqt.mediasrv.MediaServer(self)
         self.mediaServer.start()
 
     def baseHTML(self):
