@@ -1,9 +1,9 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
-# fixme: make sure not to optimize imports on this file
+# make sure not to optimize imports on this file
+# pylint: disable=unused-import
 
-import sip
 import os
 
 # fix buggy ubuntu12.04 display of language selector
@@ -14,19 +14,23 @@ from anki.utils import isWin, isMac
 from PyQt5.Qt import *
 # trigger explicit message in case of missing libraries
 # instead of silently failing to import
-from PyQt5.QtWebEngineWidgets import QWebEnginePage
+from PyQt5.QtWebEngineWidgets import *
+try:
+    from PyQt5 import sip
+except ImportError:
+    import sip
+
+from PyQt5.QtCore import pyqtRemoveInputHook # pylint: disable=no-name-in-module
 
 def debug():
-  from PyQt5.QtCore import pyqtRemoveInputHook
-  from pdb import set_trace
-  pyqtRemoveInputHook()
-  set_trace()
+    from pdb import set_trace
+    pyqtRemoveInputHook()
+    set_trace()
 
 import sys, traceback
 
 if os.environ.get("DEBUG"):
     def info(type, value, tb):
-        from PyQt5.QtCore import pyqtRemoveInputHook
         for line in traceback.format_exception(type, value, tb):
             sys.stdout.write(line)
         pyqtRemoveInputHook()
@@ -38,8 +42,9 @@ qtmajor = (QT_VERSION & 0xff0000) >> 16
 qtminor = (QT_VERSION & 0x00ff00) >> 8
 qtpoint = QT_VERSION & 0xff
 
-if ((qtmajor == 5 and qtminor == 7 and qtpoint == 0) or
-    (qtmajor == 5 and qtminor == 8 and qtpoint == 0)):
-    raise Exception("Your Qt version is known to be broken.")
-elif qtmajor <= 5 and qtminor < 7:
-    raise Exception("Anki requires Qt 5.7.1+")
+if qtmajor != 5 or qtminor < 9 or qtminor == 10:
+    raise Exception("Anki does not support your Qt version.")
+
+# GUI code assumes python 3.6+
+if sys.version_info[0] < 3 or sys.version_info[1] < 6:
+    raise Exception("Anki requires Python 3.6+")
