@@ -1,15 +1,23 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 from aqt.qt import *
 
 import time
 from anki.sound import Recorder
-from aqt.utils import saveGeom, restoreGeom
+from aqt.utils import saveGeom, restoreGeom, showWarning
+from anki.lang import _
+
+if not Recorder:
+    print("pyaudio not installed")
 
 def getAudio(parent, encode=True):
     "Record and return filename"
     # record first
+    if not Recorder:
+        showWarning("pyaudio not installed")
+        return
+
     r = Recorder()
     mb = QMessageBox(parent)
     restoreGeom(mb, "audioRecorder")
@@ -17,6 +25,7 @@ def getAudio(parent, encode=True):
     mb.setIconPixmap(QPixmap(":/icons/media-record.png"))
     but = QPushButton(_("Save"))
     mb.addButton(but, QMessageBox.AcceptRole)
+    but.setDefault(True)
     but = QPushButton(_("Cancel"))
     mb.addButton(but, QMessageBox.RejectRole)
     mb.setEscapeButton(but)
@@ -31,6 +40,7 @@ def getAudio(parent, encode=True):
         QApplication.instance().processEvents()
     if mb.clickedButton() == mb.escapeButton():
         r.stop()
+        r.cleanup()
         return
     saveGeom(mb, "audioRecorder")
     # ensure at least a second captured
