@@ -1,4 +1,4 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # -*- coding: utf-8 -*-
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
@@ -7,6 +7,7 @@ import os, time
 from aqt.utils import saveGeom, restoreGeom, maybeHideClose, addCloseShortcut, \
     tooltip, getSaveFile
 import aqt
+from anki.lang import _
 
 # Deck Stats
 ######################################################################
@@ -38,8 +39,8 @@ class DeckStats(QDialog):
         f.life.clicked.connect(lambda: self.changePeriod(2))
         maybeHideClose(self.form.buttonBox)
         addCloseShortcut(self)
-        self.refresh()
         self.show()
+        self.refresh()
         self.activateWindow()
 
     def reject(self):
@@ -78,7 +79,7 @@ class DeckStats(QDialog):
         self.refresh()
 
     def refresh(self):
-        self.mw.progress.start(immediate=True)
+        self.mw.progress.start(immediate=True, parent=self)
         stats = self.mw.col.stats()
         stats.wholeCollection = self.wholeCollection
         self.report = stats.report(type=self.period)
