@@ -1,4 +1,4 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # -*- coding: utf-8 -*-
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
@@ -6,12 +6,15 @@ from aqt.qt import *
 import aqt
 from aqt.utils import showInfo, openHelp, getOnlyText, shortcut, restoreGeom, saveGeom
 from anki.hooks import addHook, remHook
+from anki.lang import _
 
 class StudyDeck(QDialog):
     def __init__(self, mw, names=None, accept=None, title=None,
                  help="studydeck", current=None, cancel=True,
-                 parent=None, dyn=False, buttons=[], geomKey="default"):
+                 parent=None, dyn=False, buttons=None, geomKey="default"):
         QDialog.__init__(self, parent or mw)
+        if buttons is None:
+            buttons = []
         self.mw = mw
         self.form = aqt.forms.studydeck.Ui_Dialog()
         self.form.setupUi(self)
@@ -114,8 +117,6 @@ class StudyDeck(QDialog):
     def reject(self):
         saveGeom(self, self.geomKey)
         remHook('reset', self.onReset)
-        if not self.cancel:
-            return self.accept()
         QDialog.reject(self)
 
     def onAddDeck(self):
