@@ -11,7 +11,7 @@ def test_latex():
     d = getEmptyCol()
     # change latex cmd to simulate broken build
     import anki.latex
-    anki.latex.latexCmds[0][0] = "nolatex"
+    anki.latex.pngCommands[0][0] = "nolatex"
     # add a note with latex
     f = d.newNote()
     f['Front'] = "[latex]hello[/latex]"
@@ -27,7 +27,7 @@ def test_latex():
         print("aborting test; latex or dvipng is not installed")
         return
     # fix path
-    anki.latex.latexCmds[0][0] = "latex"
+    anki.latex.pngCommands[0][0] = "latex"
     # check media db should cause latex to be generated
     d.media.check()
     assert len(os.listdir(d.media.dir())) == 1
@@ -57,47 +57,28 @@ def test_latex():
     # turn it on again so other test don't suffer
     anki.latex.build = True
 
-def test_bad_latex_command_write18():
+    # bad commands
     (result, msg) = _test_includes_bad_command("\\write18")
     assert result, msg
-
-def test_bad_latex_command_readline():
     (result, msg) = _test_includes_bad_command("\\readline")
     assert result, msg
-
-def test_bad_latex_command_input():
     (result, msg) = _test_includes_bad_command("\\input")
     assert result, msg
-
-def test_bad_latex_command_include():
     (result, msg) = _test_includes_bad_command("\\include")
     assert result, msg
-
-def test_bad_latex_command_catcode():
     (result, msg) = _test_includes_bad_command("\\catcode")
     assert result, msg
-
-def test_bad_latex_command_openout():
     (result, msg) = _test_includes_bad_command("\\openout")
     assert result, msg
-
-def test_bad_latex_command_write():
     (result, msg) = _test_includes_bad_command("\\write")
     assert result, msg
-
-def test_bad_latex_command_loop():
     (result, msg) = _test_includes_bad_command("\\loop")
     assert result, msg
-
-def test_bad_latex_command_def():
     (result, msg) = _test_includes_bad_command("\\def")
     assert result, msg
-
-def test_bad_latex_command_shipout():
     (result, msg) = _test_includes_bad_command("\\shipout")
     assert result, msg
 
-def test_good_latex_command_works():
     # inserting commands beginning with a bad name should not raise an error
     (result, msg) = _test_includes_bad_command("\\defeq")
     assert not result, msg
@@ -108,7 +89,7 @@ def test_good_latex_command_works():
 def _test_includes_bad_command(bad):
     d = getEmptyCol()
     f = d.newNote()
-    f['Front'] = '[latex]%s[/latex]' % bad;
+    f['Front'] = '[latex]%s[/latex]' % bad
     d.addNote(f)
     q = f.cards()[0].q()
     return ("'%s' is not allowed on cards" % bad in q, "Card content: %s" % q)
