@@ -1,3 +1,6 @@
+/* Copyright: Ankitects Pty Ltd and contributors
+ * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */
+
 var currentField = null;
 var changeTimer = null;
 var dropTarget = null;
@@ -24,6 +27,7 @@ function saveNow(keepFocus) {
     if (keepFocus) {
         saveField("key");
     } else {
+        // triggers onBlur, which saves
         currentField.blur();
     }
 }
@@ -212,26 +216,28 @@ function caretToEnd() {
 }
 
 function onBlur() {
-    if (document.activeElement === currentField) {
-        // anki window defocused; current field unchanged
+    if (!currentField) {
         return;
     }
-    if (currentField) {
+
+    if (document.activeElement === currentField) {
+        // other widget or window focused; current field unchanged
+        saveField("key");
+    } else {
         saveField("blur");
         currentField = null;
+        disableButtons();
     }
-    clearChangeTimer();
-    disableButtons();
 }
 
 function saveField(type) {
+    clearChangeTimer();
     if (!currentField) {
         // no field has been focused yet
         return;
     }
     // type is either 'blur' or 'key'
     pycmd(type + ":" + currentFieldOrdinal() + ":" + currentNoteId + ":" + currentField.innerHTML);
-    clearChangeTimer();
 }
 
 function currentFieldOrdinal() {
@@ -336,7 +342,13 @@ function hideDupes() {
 
 var pasteHTML = function (html, internal, extendedMode) {
     html = filterHTML(html, internal, extendedMode);
-    setFormat("inserthtml", html);
+    if (html !== "") {
+        // remove trailing <br> in empty field
+        if (currentField && currentField.innerHTML === "<br>") {
+            currentField.innerHTML = "";
+        }
+        setFormat("inserthtml", html);
+    }
 };
 
 var filterHTML = function (html, internal, extendedMode) {
